'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Table = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _inheritableMap = require('inheritable-map');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Used to identify tables inside JSON blobs
 */
var TABLE_JSON_ID = '__IMMUTABLE-TABLE';

/**
 * Represents a data table in memory. Immutable
 *
 * This implementation uses nexted hash tables to represent rows/columns.  The
 * advantage of this is that a table with no data can take almost no space.  As
 * the table becomes filled, it will occupy more memory.
 */

var Table = exports.Table = function (_InheritableMap) {
  _inherits(Table, _InheritableMap);

  _createClass(Table, null, [{
    key: 'makeReviver',


    /**
     * Makes a reviver function to be used to deserialize Tables using JSON.parse.
     *
     * Serialization can be done using JSON.stringify(table)
     * @param  {function} [wrappedReviver] optionally specify another reviver
     * @return {function}
     */
    value: function makeReviver(wrappedReviver) {
      return function (key, value) {
        var newValue = value;
        if (wrappedReviver) {
          newValue = wrappedReviver(key, newValue);
        }

        if (newValue[TABLE_JSON_ID] !== undefined) {
          var object = JSON.parse(newValue[TABLE_JSON_ID]);
          var map = _immutable2.default.fromJS(object).update('data', function (data) {
            return data.mapKeys(function (x) {
              return parseInt(x, 10);
            }).map(function (values) {
              return values.mapKeys(function (y) {
                return parseInt(y, 10);
              });
            });
          });

          // Copy to Table instance
          var table = new Table(map.get('width'), map.get('height'));
          return table.set('data', map.get('data'));
        }
        return newValue;
      };
    }

    /**
     * Create a table from a CSV representation
     * @param  {string} csv comma separated value representation
     * @return {Table}
     */

  }, {
    key: 'fromCSV',
    value: function fromCSV(csv) {
      var width = 0;
      var lines = csv.split('\n').map(function (line) {
        var newLine = line.split(',').map(function (x) {
          return x.trim();
        });
        width = Math.max(width, newLine.length);
        return newLine;
      });
      var table = new Table(width, lines.length);
      lines.forEach(function (line, y) {
        line.forEach(function (item, x) {
          table = table.setCell(x, y, item);
        });
      });
      return table;
    }

    /**
     * Public constructor
     * @param  {number} width
     * @param  {number} height
     */

  }]);

  function Table(width, height) {
    var _ret;

    _classCallCheck(this, Table);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Table).call(this));

    return _ret = _this.set('width', width).set('height', height).set('data', new _immutable2.default.Map()), _possibleConstructorReturn(_this, _ret);
  }

  /**
   * The width of the table
   * @return {number}
   */


  _createClass(Table, [{
    key: 'getCell',


    /**
     * Gets a value in the table
     * @param {number} x        column index.  Negative numbers can be used to
     *                          address columns from the right side of the table.
     * @param {number} y        row index. Negative numbers can be used to address
     *                          rows from the bottom side of the table.
     * @return {any}
     */
    value: function getCell(x, y, defaultValue) {
      var _validate2 = this._validate(x, y);

      var _validate3 = _slicedToArray(_validate2, 2);

      x = _validate3[0];
      y = _validate3[1];

      return this.getIn(['data', x, y], defaultValue);
    }

    /**
     * Sets a value or values in the table.
     * @param {number} x        column index.  Negative numbers can be used to
     *                          address columns from the right side of the table.
     * @param {number} y        row index. Negative numbers can be used to address
     *                          rows from the bottom side of the table.
     * @param {any|Table} value value to set in the table.  If the value is a
     *                          table, the values of that table are used to set
     *                          the equavalent set of cells in the destination at
     *                          the offset specified.
     * @return {Table}          derived table
     */

  }, {
    key: 'setCell',
    value: function setCell(x, y, value) {
      var _this2 = this;

      var _validate4 = this._validate(x, y);

      var _validate5 = _slicedToArray(_validate4, 2);

      x = _validate5[0];
      y = _validate5[1];

      if (value instanceof Table) {
        if (value.width + x > this.width) throw new Error('table overflows x dimension');
        if (value.height + y > this.height) throw new Error('table overflows y dimension');
        return this.mergeDeep(value.delete('width').delete('height').update('data', function (data) {
          return data.mapKeys(function (fromX) {
            return fromX + x;
          }).filter(function (_, fromX) {
            return fromX < _this2.width;
          }).map(function (map) {
            return map.mapKeys(function (fromY) {
              return fromY + y;
            }).filter(function (_, fromY) {
              return fromY < _this2.height;
            });
          });
        }));
      }
      return this.setIn(['data', x, y], value);
    }

    /**
     * Get a subset of the table
     * @param  {number} startX the left most column index of the subset.  Negative
     *                         numbers can be used to address columns from the
     *                         right side of the table.
     * @param  {number} startY the top most row of the subset.  Negative numbers
     *                         can be used to address rows from the bottom side of
     *                         the table.
     * @param  {number} [endX] the right most column index, exclusive, of the
     *                         subset.  Negative numbers can be used to address
     *                         columns from the right side of the table.
     * @param  {number} [endY] the bottom most row index, exclusive, of the
     *                         subset.  Negative numbers can be used to address
     *                         rows from the bottom side of the table.
     * @return {Table}         subset
     */

  }, {
    key: 'slice',
    value: function slice(startX, startY, endX, endY) {

      // If a right isn't defined, use the right edge and bottom edge for x/y lims

      var _validate6 = this._validate(startX, startY);

      var _validate7 = _slicedToArray(_validate6, 2);

      startX = _validate7[0];
      startY = _validate7[1];
      if (endX === undefined) {
        endX = this.width;
        endY = this.height;
      } else {
        var _validate8 = this._validate(endX, endY);

        var _validate9 = _slicedToArray(_validate8, 2);

        endX = _validate9[0];
        endY = _validate9[1];
      }

      return this.set('width', endX - startX).set('height', endY - startY).update('data', function (data) {
        return data.filter(function (values, fromX) {
          return startX <= fromX && fromX < endX;
        }).mapKeys(function (fromX) {
          return fromX - startX;
        }).map(function (map) {
          return map.filter(function (values, fromY) {
            return startY <= fromY && fromY < endY;
          }).mapKeys(function (fromY) {
            return fromY - startY;
          });
        });
      });
    }

    /**
     * Serialize the table to a JSON representation.
     *
     * Deserialization can be done using the static Table.makeReviver() function
     * in combination with JSON.parse();
     * @return {string} JSON representation
     */

  }, {
    key: 'toJSON',
    value: function toJSON() {
      var serialized = {};
      serialized[TABLE_JSON_ID] = JSON.stringify(this.toJS());
      return serialized;
    }

    /**
     * Converts negative indicies into positive ones and also validates indices.
     *
     * Throws if coordinates are invalid.
     * @param  {number} x column index, can be negative
     * @param  {number} y row index, can be negative
     * @return {number[]} tuple of x, y indicies
     */

  }, {
    key: '_validate',
    value: function _validate(x, y) {
      if (x < 0) x += this.width;
      if (y < 0) y += this.height;
      if (x >= this.width) throw new Error('x larger than table width');
      if (y >= this.height) throw new Error('y larger than table height');
      if (x < 0) throw new Error('x less than 0');
      if (y < 0) throw new Error('y less than 0');
      return [x, y];
    }

    /**
     * Create a friendly string representation of the Table
     * @return {string}
     */

  }, {
    key: 'toString',
    value: function toString() {
      return this.__toString('Table {', '}');
    }
  }, {
    key: 'width',
    get: function get() {
      return this.get('width');
    }

    /**
     * The height of the table
     * @return {number}
     */

  }, {
    key: 'height',
    get: function get() {
      return this.get('height');
    }
  }]);

  return Table;
}(_inheritableMap.InheritableMap);